Program cleanup;

{ This program "cleans up" the source code of your Turbo-Pascal
applications before printing/uploading. It passes through your
source code once, converting all words (except comments and
literals statements) to lower case and capitalizing the reserved
words of Turbo Pascal.

Thanks go to Bill Cote and J.W. Kindschi, Jr. for their programs
LOWCASE.PAS and TURBFIX.PAS (respectively). These two programs
were combined with some additional code to get to what you
see in this program.

A final note: this program is valid for TURBO version 3.0, and
has been tested on my CPM-80 system. I assume it will work on
MSDOS, CPM86 and others. Please help out with revisions, etc.
and keep me posted.

7/31/85, Doug Pearson [75366,2413]

Program Modification History
  Date    Author                  Changes made
09/14/85  Roy J. Collins     1. Made KEY array dynamically allocate space
                                for each entry. Array is now permanently
                                allocated at 244 entries (the max needed).
                                Space is used on an as-needed basis.
                             2. Sort KEY array after loading.
                             3. Implement binary search for speed
                             4. Added recognition of (* for comment as
                                well as {.
                             5. Added the underscore character (_) to the
                                characters allowed in a word.
                             6. Made it a single pass process - it used to
                                require two passes.
                             7. Allow more than one file to be processed
                                during one execution of the program.
                             8. Added array UP_KEY to hold upper case version
                                of KEY[]; UP_KEY is used for fast searching;
                                KEY is used to retrieve the properly
                                capitalized key word.
                             9. Non-reserved words do not change case.
                                (See procedure 'process_word' to place these
                                words in lower case).


The following constants must be set properly for your system:
      OP_SYSTEM  Must be set to CPM80, CPM86, or MSDOS
      HAS_GRAPHICS
      HAS_EXTENDED_GRAPHICS
      HAS_BCD    These must be set TRUE or FALSE as appropriate.
}

Type
     computers= (cpm80,cpm86,MsDos);
Const
     op_system             : computers = MsDos;
     has_graphics          : Boolean   = True;
     has_extended_graphics : Boolean   = False;
     has_bcd               : Boolean   = False;

     basic_keywords   = 158;
     maximum_keywords = 244; {max allowed}

Type
  name  = String[14];
  cmd   = (read_access,write_access);
  alpha = String[135];

Var
  line_number,
  keywords_used : Integer;
  infile,
  outfile       : Text;
  ch            : Char;
  line          : alpha;
  source,
  dest          : name;
  key           : Array[1..maximum_keywords] Of ^alpha;
  up_key        : Array[1..maximum_keywords] Of ^alpha;

Function up_string(s:alpha):alpha;
Var
  i : Integer;
Begin
  For i := 1 To Length(s) Do
    s[i] := UpCase(s[i]);
  up_string := s;
End; (* func up_string *)

Procedure sort;
Var
  hold : ^alpha;
  d,i,j,x : Integer;
Begin
  Write('sorting keywords...');
  d := 1 Shl (Trunc(Ln(keywords_used) / Ln(2)) - 1);
  Repeat
    For i := 1 To keywords_used-d Do Begin
      j := i;
      While j >= 1 Do Begin
        x := j+d;
        If up_key[j]^ > up_key[x]^ Then Begin
          hold := key[x];
          key[x] := key[j];
          key[j] := hold;
          hold := up_key[x];
          up_key[x] := up_key[j];
          up_key[j] := hold;
          End;
        j := j - d;
        End; (* while *)
      End; (* for *)
    d := d Shr 1; (* divide by 2 *)
  Until d <= 0;
  Writeln;
End; (* proc sort *)

Function search(id:alpha):Integer;
Var
  quit : Boolean;
  top,bottom,mid : Integer;
  i: Integer;
Begin
  For i:=1 To Length(id) Do
    id[i]:=UpCase(id[i]);
  bottom := 1;
  top    := keywords_used;
  quit   := False;
  search := 0;
  Repeat
    mid := ((top-bottom) Shr 1) + bottom;
    If ((mid = bottom) Or (mid = top)) Then
      quit := True
    Else
      If id > up_key[mid]^ Then
        bottom := mid
      Else
      If id < up_key[mid]^ Then
        top := mid
      Else Begin
        quit := True;
        search := mid;
        End;
  Until quit;
End; (* func search *)

Procedure initkeys;
Var
  sub : Integer;

  Procedure add_key(k:alpha);
  Begin
    sub := sub + 1;
    New(key[sub]);
    key[sub]^ := k;
    New(up_key[sub]);
    up_key[sub]^ := up_string(k);
  End; (* proc add_key *)

Begin
  Write('Loading keywords...');
  sub := 0;
  add_key('Abs');       add_key('Absolute');   add_key('Addr');
  add_key('And');       add_key('Append');     add_key('ArcTan');
  add_key('Array');     add_key('Assign');     add_key('Aux');
  add_key('AuxInPtr');  add_key('AuxOutPtr');  add_key('Begin');
  add_key('BlockRead'); add_key('BlockWrite'); add_key('Boolean');
  add_key('BufLen');    add_key('Byte');       add_key('Case');
  add_key('Chain');     add_key('Char');       add_key('Chr');
  add_key('Close');     add_key('ClrEol');     add_key('ClrScr');
  add_key('Con');       add_key('Concat');     add_key('ConInPtr');
  add_key('ConOutPtr'); add_key('Const');      add_key('ConstPtr');
  add_key('Copy');      add_key('Cos');        add_key('CrtExit');
  add_key('CrtInit');   add_key('Delay');      add_key('Delete');
  add_key('DelLine');   add_key('Dispose');    add_key('Div');
  add_key('Do');        add_key('DownTo');     add_key('Else');
  add_key('End');       add_key('Eof');        add_key('Eoln');
  add_key('Erase');     add_key('Execute');    add_key('Exit');
  add_key('Exp');       add_key('External');   add_key('False');
  add_key('File');      add_key('FilePos');    add_key('FileSize');
  add_key('FillChar');  add_key('Flush');      add_key('For');
  add_key('Forward');   add_key('Frac');       add_key('FreeMem');
  add_key('Function');  add_key('GetMem');     add_key('GoTo');
  add_key('GotoXY');    add_key('Halt');       add_key('HeapPtr');
  add_key('Hi');        add_key('If');         add_key('In');
  add_key('Inline');    add_key('Input');      add_key('Insert');
  add_key('InsLine');   add_key('Int');        add_key('Integer');
  add_key('IoResult');  add_key('Kbd');        add_key('KeyPressed');
  add_key('Label');     add_key('Length');     add_key('Ln');
  add_key('Lo');        add_key('LowVideo');   add_key('Lst');
  add_key('LstOutPtr'); add_key('Mark');       add_key('MaxAvail');
  add_key('MaxInt');    add_key('Mem');        add_key('MemAvail');
  add_key('MemW');      add_key('Mod');        add_key('Move');
  add_key('New');       add_key('Nil');        add_key('NormVideo');
  add_key('Not');       add_key('Odd');        add_key('Of');
  add_key('Or');        add_key('Ord');        add_key('Output');
  add_key('Overlay');   add_key('Packed');     add_key('ParamCount');
  add_key('ParamStr');  add_key('PI');         add_key('Port');
  add_key('Pos');       add_key('Pred');       add_key('Procedure');
  add_key('Program');   add_key('Ptr');        add_key('Random');
  add_key('Randomize'); add_key('Read');       add_key('ReadLn');
  add_key('Real');      add_key('Record');     add_key('Release');
  add_key('Rename');    add_key('Repeat');     add_key('Reset');
  add_key('ReWrite');   add_key('Round');      add_key('Seek');
  add_key('SeekEof');   add_key('SeekEoLn');   add_key('Set');
  add_key('Shl');       add_key('Shr');        add_key('Sin');
  add_key('SizeOf');    add_key('Sqr');        add_key('Sqrt');
  add_key('Str');       add_key('String');     add_key('Succ');
  add_key('Swap');      add_key('Text');       add_key('Then');
  add_key('To');        add_key('TRM');        add_key('True');
  add_key('Trunc');     add_key('Type');       add_key('Until');
  add_key('UpCase');    add_key('USR');        add_key('UsrInPtr');
  add_key('UsrOutPtr'); add_key('Val');        add_key('Var');
  add_key('While');     add_key('With');       add_key('Write');
  add_key('Writeln');   add_key('Xor');

  Case op_system Of
    cpm80: Begin
           add_key('Bdos');       add_key('BdosHL');
           add_key('Bios');       add_key('BiosHL');
           add_key('OvrDrive');   add_key('RecurPtr');
           add_key('StackPtr');
           End;
    cpm86: Begin
           add_key('Bdos');       add_key('Bios');
           add_key('CSeg');       add_key('DSeg');
           add_key('Intr');       add_key('MemW');
           add_key('Ofs');        add_key('OvrDrive');
           add_key('PortW');      add_key('Seg');
           add_key('SSeg');
           End;
    MsDos: Begin
           add_key('ChDir');          add_key('CSeg');
           add_key('DSeg');           add_key('GetDir');
           add_key('Intr');           add_key('LongFilePos');
           add_key('LongFileSize');   add_key('LongSeek');
           add_key('MemW');           add_key('MkDir');
           add_key('MsDos');          add_key('Ofs');
           add_key('OvrPath');        add_key('PortW');
           add_key('RmDir');          add_key('Seg');
           add_key('SSeg');           add_key('Truncate');
           If has_bcd Then
             add_key('Form');
           End;
  End; {case of op_system}

  If ((op_system=MsDos) And (has_graphics)) Then Begin
    add_key('Black');        add_key('Blink');           add_key('Blue');
    add_key('Brown');        add_key('Cyan');            add_key('DarkGray');
    add_key('Draw');         add_key('GraphBackGround'); add_key('GraphColorMode');
    add_key('GraphMode');    add_key('GraphWindow');     add_key('Green');
    add_key('HiRes');        add_key('HiResColor');      add_key('LightBlue');
    add_key('LightCyan');    add_key('LightGray');       add_key('LightGreen');
    add_key('LightMagenta'); add_key('LightRed');        add_key('Magenta');
    add_key('NoSound');      add_key('Palette');         add_key('Plot');
    add_key('Red');          add_key('Sound');           add_key('TextBackGround');
    add_key('TextColor');    add_key('TextMode');        add_key('WhereX');
    add_key('WhereY');       add_key('White');           add_key('Window');
    add_key('Yellow');
    If has_extended_graphics Then Begin
      add_key('Arc');         add_key('Back');         add_key('Circle');
      add_key('ClearScreen'); add_key('ColorTable');   add_key('East');
      add_key('FillPattern'); add_key('FillScreen');   add_key('FillShape');
      add_key('GetDotColor'); add_key('GetPic');       add_key('Heading');
      add_key('HideTurtle');  add_key('Home');         add_key('North');
      add_key('NoWrap');      add_key('Pattern');      add_key('PenDown');
      add_key('PenUp');       add_key('PutPic');       add_key('SetHeading');
      add_key('SetPenColor'); add_key('SetPosition');  add_key('ShowTurtle');
      add_key('South');       add_key('TurnLeft');     add_key('TurnRight');
      add_key('TurtleThere'); add_key('TurtleWindow'); add_key('West');
      add_key('Wrap');        add_key('XCor');         add_key('YCor');
      End; {extended graphics}
  End; {regular graphics}
  keywords_used := sub;
  Writeln;
End; {initkeys}

Function exists(filename: name; func: cmd): Boolean;
Begin
  {$I-}
  If func = read_access Then Begin
    Assign(infile,filename);
    Reset(infile);
    End
  Else Begin
    Assign(outfile,filename);
    ReWrite(outfile);
    End;
  {$I+}
  exists := IoResult = 0;
End; (* func exists *)

Procedure lowercase(Var Str:alpha);
Var
  i : Integer;
Begin
  For i := 1 To Length(Str) Do
    If ((Str[i] >= 'A') And (Str[i] <= 'Z')) Then
      Str[i] := Chr(Ord(Str[i]) Or $20);
End;

Procedure convert_text;
Var
  ch : Char;
  id : alpha;
  lptr : Integer;

  Procedure write_read;
  Begin
    Writeln(outfile,line);
    ReadLn(infile,line);
    If line_number Mod 16 = 0 Then Begin
      GotoXY(1,11);
      Write(line_number:5);
      End;
    line_number := line_number + 1;
  End; (* proc write_read *)

  Procedure skip_comment;
  Var
    delim : String[2];
    s,p : Integer;
  Begin
    If line[lptr] = '{' Then
      delim := '}'
    Else
      delim := '*)';
    s := lptr+Length(delim);
    p := Pos(delim,Copy(line,s,999));
    If p > 0 Then
      lptr := s + p + Length(delim)
    Else Begin
      Repeat
        write_read;
        lptr := Pos(delim,line);
      Until lptr > 0;
      lptr := lptr + Length(delim);
      End;
  End; (* proc skip_comment *)

  Procedure process_word;
  Var
    xx,
    wptr : Integer;
  Begin
    wptr:=1; id:='';
    ch := line[lptr];
    While ((ch>='a') And (ch<='z')) Or
          ((ch>='A') And (ch<='Z')) Or
          ((ch>='0') And (ch<='9')) Or
          (ch='_') Do Begin
      wptr := wptr + 1;
      ch := line[lptr+wptr-1];
      End;
    id := Copy(line,lptr,wptr-1);
    xx := search(id);
    If xx > 0 Then
      Move(key[xx]^[1], line[lptr], Length(id));
    {***** Following code may be enabled to convert non-reserved words to
           all lower case:
    else begin
      lowercase(id);
      move(id[1], line[lptr], length(id);
      end;
    **********************************************************************}
    lptr:=lptr+wptr;
  End; (* proc process_word *)

Begin
  Write('Converting/capitalizing...');
  line_number := 0;
  ReadLn(infile,line);
  While ((Not Eof(infile)) Or (line<>'')) Do Begin
    If line <> '' Then Begin
      lptr:=1;
      While lptr<=Length(line) Do Begin
        If ((line[lptr] = '{') Or
           ((line[lptr]='(') And (line[lptr+1]='*') And
           (lptr<Length(line)))) Then
          skip_comment;
        If line[lptr] = '''' Then
          lptr := Pos('''',Copy(line,lptr+1,999))+lptr+1;
        ch := line[lptr];
        If ((ch >= 'a') And (ch <= 'z')) Or
           ((ch >= 'A') And (ch <= 'Z')) Then
          process_word
        Else
          lptr:=lptr+1;
        End; {while lptr}
      End; {<>''}
    write_read;
    End; {eof test}
  Writeln;
End; {convert_text}

Procedure init;
Begin
  ClrScr;
  initkeys;
  sort;
  Writeln;
  Writeln('This program converts upper to lower case');
  Writeln('and capitalizes reserved words');
  Writeln;
End; (* proc init *)

Procedure get_files;
Begin
  Repeat
    GotoXY(1,7);
    ClrEol;
    Write('Input File: ');
    ReadLn(source);
  Until exists(source,read_access);
  Repeat
    GotoXY(1,9);
    ClrEol;
    Write('Destination File: ');
    ReadLn(dest);
  Until dest <> source;
End; (* proc get_files *)

Begin {cleanup}
  init;
  Repeat
    get_files;
    If exists(dest,write_access) Then Begin
      convert_text;
      GotoXY(1,11);
      Write(line_number:5);
      End
    Else
      Writeln('Error opening ',dest);
    {$I-}
    Close(infile);
    Close(outfile);
    {$I+}
    Writeln;
    Writeln('Cleanup now complete.');
    Writeln;
    Write('Cleanup another file? (Y/N) ');
    Repeat
      Read(Kbd,ch);
      ch := UpCase(ch);
    Until ((ch = 'Y') Or (ch = 'N'));
    Writeln(ch);
    If ch = 'Y' Then
      ClrScr;
    Until ch = 'N';
End.
